package com.jiuqi.util.zip;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.jiuqi.util.zip.AsiExtraField;
import com.jiuqi.util.zip.UnrecognizedExtraField;
import com.jiuqi.util.zip.ZipExtraField;
import com.jiuqi.util.zip.ZipRuntimeException;
import com.jiuqi.util.zip.ZipShort;
import java.util.Hashtable;
import java.util.Vector;
import java.util.zip.ZipException;

public class ExtraFieldUtils {
  private static Hashtable implementations = new Hashtable<Object, Object>();
  
  static {
    AsiExtraField field = new AsiExtraField();
    register(field.getClass());
  }
  
  public static void register(Class<ZipExtraField> c) {
    try {
      ZipExtraField ze = c.newInstance();
      implementations.put(ze.getHeaderId(), c);
    } catch (ClassCastException classCastException) {
      throw new ZipRuntimeException(c + " doesn't implement ZipExtraField");
    } catch (InstantiationException instantiationException) {
      throw new ZipRuntimeException(c + " is not a concrete class");
    } catch (IllegalAccessException illegalAccessException) {
      throw new ZipRuntimeException(c + "'s no-arg constructor is not public");
    } 
  }
  
  public static ZipExtraField createExtraField(ZipShort headerId) throws InstantiationException, IllegalAccessException {
    Class<ZipExtraField> c = (Class)implementations.get(headerId);
    if (c != null)
      return c.newInstance(); 
    UnrecognizedExtraField u = new UnrecognizedExtraField();
    u.setHeaderId(headerId);
    return (ZipExtraField)u;
  }
  
  public static ZipExtraField[] parse(byte[] data) throws ZipException {
    Vector<ZipExtraField> v = new Vector();
    int start = 0;
    while (start <= data.length - 4) {
      ZipShort headerId = new ZipShort(data, start);
      int length = (new ZipShort(data, start + 2)).getValue();
      if (start + 4 + length > data.length)
        throw new ZipException("data starting at " + start + 
            " is in unknown format"); 
      try {
        ZipExtraField ze = createExtraField(headerId);
        ze.parseFromLocalFileData(data, start + 4, length);
        v.addElement(ze);
      } catch (InstantiationException ie) {
        throw new ZipException(ie.getMessage());
      } catch (IllegalAccessException iae) {
        throw new ZipException(iae.getMessage());
      } 
      start += length + 4;
    } 
    if (start != data.length)
      throw new ZipException("data starting at " + start + 
          " is in unknown format"); 
    ZipExtraField[] result = new ZipExtraField[v.size()];
    v.copyInto((Object[])result);
    return result;
  }
  
  public static byte[] mergeLocalFileDataData(ZipExtraField[] data) {
    int sum = 4 * data.length;
    for (int i = 0; i < data.length; i++)
      sum += data[i].getLocalFileDataLength().getValue(); 
    byte[] result = new byte[sum];
    int start = 0;
    for (int j = 0; j < data.length; j++) {
      System.arraycopy(data[j].getHeaderId().getBytes(), 
          0, result, start, 2);
      System.arraycopy(data[j].getLocalFileDataLength().getBytes(), 
          0, result, start + 2, 2);
      byte[] local = data[j].getLocalFileDataData();
      System.arraycopy(local, 0, result, start + 4, local.length);
      start += local.length + 4;
    } 
    return result;
  }
  
  public static byte[] mergeCentralDirectoryData(ZipExtraField[] data) {
    int sum = 4 * data.length;
    for (int i = 0; i < data.length; i++)
      sum += data[i].getCentralDirectoryLength().getValue(); 
    byte[] result = new byte[sum];
    int start = 0;
    for (int j = 0; j < data.length; j++) {
      System.arraycopy(data[j].getHeaderId().getBytes(), 
          0, result, start, 2);
      System.arraycopy(data[j].getCentralDirectoryLength().getBytes(), 
          0, result, start + 2, 2);
      byte[] local = data[j].getCentralDirectoryData();
      System.arraycopy(local, 0, result, start + 4, local.length);
      start += local.length + 4;
    } 
    return result;
  }
}
